/**
 * Method: We want a structure that can represent the interval [x...y] as the
 * key-value pairs (x, +1) and (y + 1, -1). Then the answer for a query at x
 * is the sum of the values for keys <= x.
 *
 * We use an augmented skip list for every line. Each pointer in the skip list
 * stores the sum of the values underneath it, called delta. We use the deltas
 * to quickly compute the sum above as we look up x.
 *
 * Since line numbers can be huge, we also use an index skip list to map line
 * numbers to skip lists.
 **/
#include <stdio.h>
#include <stdlib.h>

/* number of queries, also equal to the number of lines */
#define MAX_Q 500000

/* highest level we allow for skip lists */
#define MAX_LEVEL 13
#define INF 0x7fffffffffffffffull

/* syntactic sugar for the target and delta of */
#define CHILD(cell, num) (a[c[(cell)].child + (num)].node)
#define DELTA(cell, num) (a[c[(cell)].child + (num)].delta)

typedef struct {
  long long key;   /* key being stored */
  int child;       /* index in c of the pointer list */
  char h;          /* height (pointer count) */
} cell;

typedef struct {
  int node;        /* target of the pointer */
  int delta;       /* difference between [ and ] underneath the arrow */
} arrow;

/* Preallocated cells in all skip lists. */
/* We need MAX_Q sentinels and we my have to insert 2 * MAX_Q keys */
cell c[MAX_Q * 3];

/* Preallocated arrows. */
arrow a[MAX_Q * MAX_LEVEL + MAX_Q * 21/10];

int list[MAX_Q];        /* skip list heads */
int cPtr, aPtr;         /* pointers to the next available elements in c and a */
int inc[MAX_LEVEL];     /* using during insert() */

/* Creates a new, empty skip list and return its head's position in c */
int makeList() {
  /* Create a head with key -INF and MAX_LEVEL pointers. */
  /* Arrows will implicitly have delta = 0 and point to the tail. */
  int result = cPtr;
  c[cPtr].key = -INF;
  c[cPtr].h = MAX_LEVEL;
  c[cPtr].child = aPtr;
  cPtr++;
  aPtr += MAX_LEVEL;
  return result;
}

/* Chooses a height for a newly inserted node. */
int randomLevel() {
  int l = 1;
  int r = rand() & ~(1 << (MAX_LEVEL - 1));
  while (r & 1) {
    l++;
    r >>= 1;
  }
  return l;
}

/* Inserts the given key and value in the given list. */
/* If the key already exists in the list, then the value is updated. */
void insert(int head, long long key, int value) {
  int pos = head, l;
  for (l = MAX_LEVEL - 1; l >= 0; l--) {
    while (c[CHILD(pos, l)].key < key) {
      pos = CHILD(pos, l);
    }
    inc[l] = pos;
  }

  pos = CHILD(pos, 0);
  if (c[pos].key == key) {
    /* Existing node: update the deltas on incoming pointers. */
    for (l = 0; l < c[pos].h; l++) {
      DELTA(inc[l], l) += value;
    }
  } else {
    /* Insert a new node. */
    pos = cPtr++;
    c[pos].key = key;
    c[pos].h = randomLevel();
    c[pos].child = aPtr;

    /* Redo the pointers on every level. */
    for (l = 0; l < c[pos].h; l++) {
      a[aPtr++].node = CHILD(inc[l], l);
      CHILD(inc[l], l) = pos;
    }

    /* Update the deltas to our right. */
    DELTA(pos, 0) = DELTA(inc[0], 0);
    DELTA(inc[0], 0) = value;
    int next = CHILD(pos, 0);
    int d = DELTA(pos, 0);
    for (l = 1; l < c[pos].h; l++) {
      while (next != a[c[pos].child + l].node) {
        d += DELTA(next, l - 1);
        next = CHILD(next, l - 1);
      }
      DELTA(pos, l) = d;
      DELTA(inc[l], l) += value - d;
    }
  }

  /* Pointers on higher levels go above us, so they gain +value. */
  for (l = c[pos].h; l < MAX_LEVEL; l++) {
    DELTA(inc[l], l) += value;
  }
}

/* Searches for the key in the given list. Sums the deltas as it goes along. */
int search(int head, long long key) {
  int result = 0;
  int pos = head, l;
  for (l = MAX_LEVEL - 1; l >= 0; l--) {
    while (c[CHILD(pos, l)].key <= key) {
      result += DELTA(pos, l);
      pos = CHILD(pos, l);
    }
  }
  return result;
}

/* Searches the given key in the index list. If not found, creates a new */
/* skip list and returns its position. */
int searchIndex(long long key) {
  int pos = 1, l;
  for (l = MAX_LEVEL - 1; l >= 0; l--) {
    while (c[CHILD(pos, l)].key < key) {
      pos = CHILD(pos, l);
    }
    inc[l] = pos;
  }

  pos = CHILD(pos, 0);
  if (c[pos].key != key) {
    /* Insert a new node. */
    pos = cPtr++;
    c[pos].key = key;
    c[pos].h = randomLevel();
    c[pos].child = aPtr;

    /* Redo the pointers on every level. */
    for (l = 0; l < c[pos].h; l++) {
      a[aPtr++].node = CHILD(inc[l], l);
      CHILD(inc[l], l) = pos;
    }

    DELTA(pos, 0) = makeList();
  }
  return DELTA(pos, 0);
}

int ll = 0;

void initialize(long long n, long long m) {
  /* Create one (common) tail with key = +INF and zero height. */
  c[cPtr++].key = +INF;

  /* c[1] will store the head of the index list */
  makeList();
}

void update(long long x, long long y1, long long y2) {
  int l = searchIndex(x);
  insert(l, y1, 1);
  insert(l, y2 + 1, -1);
}

int query(long long x, long long y) {
  int l = searchIndex(x);
  return search(l, y);
}
